@@ -976,6 +976,10 @@ int ffintfstr(int *input,      /* I - array of values to be converted  */
 {
     long ii;
     double dvalue;
+    char *cptr;
+    
+    cptr = output;
+
 
     if (scale == 1. && zero == 0.)
     {       
@@ -1000,5 +1004,9 @@ int ffintfstr(int *input,      /* I - array of values to be converted  */
             *status = OVERFLOW_ERR;
         }
     }
+
+    /* replace any commas with periods (e.g., in French locale) */
+    while ((cptr = strchr(cptr, ','))) *cptr = '.';
+    
     return(*status);
 }
